# OpenXT

> Use Open Standards, Open Data, Open Source, and Open Innovation

A portable, simple, cross-platform, fast implementation of CoreXT. 

Download release at <https://drive.recolic.net/s/CbtmdPNo6dStX6W> or <https://git.recolic.net/msc/openxt/-/pipelines>. (Azure Pipeline artifact uploading is too hard to use)

## Dependency

> This is a naive new Go programmer. The coding style is bad and don't learn from it!

Build time: Go 

Runtime: A computer or something looks like a computer

## Build

```bash
go build .
```

## Usage

#### 1. Add MSCORP sources to your nuget.config on linux machine. 
   
To avoid the annoying login, goto `DevOps -> TopRightCorner -> Settings -> Personal Access Tokens`, 
then create a new token, select `All accessible organizations`, and allow permission `Packaging/read`. 
Save the token for future usage. 

> It's ok to use `nuget` instead of dotnet 5.0. Just give a valid `nuget.config` and openxt is happy!

Then use `dotnet nuget add` to add all sources your need. This is an example: 

```bash
dotnet nuget add source "https://msazure.pkgs.visualstudio.com/_packaging/Official/nuget/v3/index.json" --name MSAzure --username YOUR_ALIAS --password "xbwejup___Your_DevOps_PAT_Token_Here___73ew6iq" --store-password-in-clear-text
```

Refer to `corext.config` for repo URLs used by your project.

After that, make sure `~/.nuget/NuGet/NuGet.Config` got populated correctly. 

#### 2. Use OpenXT to init CxCache

`CxCache` is a bad name, let's name it as `Local Nuget Repo`. 

Init or sync your `local nuget repo` with the following command: 

```bash
# load package list from csproj files
openxt sync --project-dir "~/code/Azure-Deployment-Builder" --local-repo-dir "/mnt/hdd/CxCache"
# or load from packages.config
openxt sync2 --project-dir "~/code/vfp/windows" --local-repo-dir "/mnt/hdd/CxCache"
```

Refer to FAQ to learn the difference between sync/sync2.

> It's OK for multiple projects to share one local-repo-dir. 

> It's OK to set --project-dir to a small sub-project, instead of the huge ControlPlane. 

#### 3. Use OpenXT to develop or build your project (C-Sharp project only)

> There's outdated syntax in many csproj files, which prevents your project to build in .NET 5. 
> This IS NOT a OpenXT issue, read [doc/csproj-compliance-net5.md](doc/csproj-compliance-net5.md) for more detail. 

OpenXT currently supports bash, fish, and ~~powershell~~. You may run the following command to enable your
shell session to work on CoreXT projects: 

```bash
# bash-only
openxt env --shell bash --project-dir "~/code/Azure-Deployment-Builder" --local-repo-dir "/mnt/hdd/CxCache" | source
rider xxx.sln
```

```bash
# fish-only
openxt env --shell fish --project-dir "~/code/Azure-Deployment-Builder" --local-repo-dir "/mnt/hdd/CxCache" | source
dotnet build
```

## Performance

In our Azure-Deployment-Builder repo, using Windows Bare-Metal (1065G7, 16G-DDR4-3733MHz, NVME SSD) and 
Linux Bare-Metal (W-2123, 32G-DDR4-2666MHz, SATA SSD). Windows defender has been destroyed on the CoreXT machine. 

|Time|CoreXT (Windows Bare-Metal)|OpenXT (Linux Bare-Metal)|
|:---|:---:|:---:|
|First launch (init)|10.83min|10.57s|
|Subsequent launch|1.435s(cache) - 20.998s(no_cache)|31.86ms|

## TODO

1. deps de-duplicate

2. possible coreXT design issue: If `myProject/subproject1` requires pkgA:net472, 
   and `myProject/subproject2` requires pkgA:net451. And pkgA:1.0.1 only provides net451, 
   and pkgA:1.0.2 only provides net472, it booms.

## FAQ

- What's the difference of sync / sync2?

`openxt sync` parse package list from csproj files, which can be inaccurate, and unuseable for non-csharp project.

`openxt sync2` parse package list from `packages.config`, which is more universal and reliable. But some team didn't maintain it correctly.

If your package list are maintained in `corext.config`, just extract `packages.config` with `sed -n '/<packages>/,/<\/packages>/p' $filename`.

- How do I build ControlPlane with OpenXT?

OpenXT is not a build management tool, and we use dotnet 5.0 to build projects. 
One software should do one thing, and there would be another tool to use `dotnet build` to build a 
large directory like ControlPlane. 

It's easier to only build the small project that you're working on. 

- OpenXT-built CIS workflow fails to run on CIS beta site.

CIS isn't fully happy with .NET 5. You should do the following manually: 

1. Make sure you have copied *.xaml to the output directory. 
2. You must delete all *.pdb file from the output directory. (or build a release/retail version instead of debug version)

- Powershell not working

Powershell sucks. Please use bash for windows. (MinGW bash is enough). 

```bash
openxt env --local-repo-dir C:/oxtrepo --bin-dir C:/oxtbin --project-dir . > env.log
source env.log
"C:\Program Files (x86)\Microsoft Visual Studio\2019\Enterprise\Common7\IDE\devenv.exe" M365FleetAGC.sln
```

- Strange build error: dotnet is using old version of package, even if I have modified

Try to delete the local cache directory: `packages`. 

## Bug report

If you are not able to open an issue, feel free to email bensl@microsoft.com to report a bug. 
